require 'dread'

module Dread
  class GadgetEvent < Event
    attr_reader    :link
    attr_reader    :type

    LINK_STATE_U0            = 0
    LINK_STATE_U1            = 1
    LINK_STATE_U2            = 2
    LINK_STATE_U3            = 3
    LINK_STATE_SS_DIS        = 4
    LINK_STATE_RX_DET        = 5
    LINK_STATE_SS_INACT      = 6
    LINK_STATE_POLL          = 7
    LINK_STATE_RECOV         = 8
    LINK_STATE_HRESET        = 9
    LINK_STATE_CMPLY         = 10
    LINK_STATE_LPBK          = 11
    LINK_STATE_RESET         = 14
    LINK_STATE_RESUME        = 15

    EVENT_DISCONNECT         = 0
    EVENT_RESET              = 1
    EVENT_CONNECT_DONE       = 2
    EVENT_LINK_STATE_CHANGE  = 3
    EVENT_WAKEUP             = 4
    EVENT_HIBER_REQ          = 5
    EVENT_EOPF               = 6
    EVENT_SOF                = 7
    EVENT_L1SUSP             = 8
    EVENT_ERRATIC_ERROR      = 9
    EVENT_CMD_CMPL           = 10
    EVENT_OVERFLOW           = 11
    EVENT_VNDRDEVTSTRCVED    = 12
    EVENT_STOPPEDONDISCONNECT= 13
    EVENT_L1RESUME           = 14

    def initialize(line)
      super

      @event_type = {
        EVENT_DISCONNECT => "Disconnect",
        EVENT_RESET => "Reset",
        EVENT_CONNECT_DONE => "Connect Done",
        EVENT_LINK_STATE_CHANGE => "Link State Change",
        EVENT_WAKEUP => "Wakeup",
        EVENT_HIBER_REQ => "Hibernation Request",
        EVENT_EOPF => "End Of Packet Frame",
        EVENT_SOF => "Start Of Frame",
        EVENT_L1SUSP => "L1 Suspend",
        EVENT_ERRATIC_ERROR => "Erratic Error",
        EVENT_CMD_CMPL => "Command Complete",
        EVENT_OVERFLOW => "Overflow",
        EVENT_VNDRDEVTSTRCVED => "Vendor Device Test LMP Received",
        EVENT_STOPPEDONDISCONNECT => "Stopped On Disconnect",
        EVENT_L1RESUME => "L1 Resume / Remote Wake",
      }

      @link_state = {
        LINK_STATE_U0 => "U0",
        LINK_STATE_U1 => "U1",
        LINK_STATE_U2 => "U2",
        LINK_STATE_U3 => "U3",
        LINK_STATE_SS_DIS => "SS.Disconnect",
        LINK_STATE_RX_DET => "Rx.Detect",
        LINK_STATE_SS_INACT => "SS.Inactive",
        LINK_STATE_POLL => "Polling",
        LINK_STATE_RECOV => "Recovery",
        LINK_STATE_HRESET => "Hot Reset",
        LINK_STATE_CMPLY => "Compliance",
        LINK_STATE_LPBK => "Loopback",
        LINK_STATE_RESET => "Reset",
        LINK_STATE_RESUME => "Resume",
      }

      decode
    end

    def decode
      @event_data = super

      @type = field(@event_data, 11, 8)
      @link = field(@event_data, 19, 16)
    end

    def to_s
      super + ": #{@event_type[@type]} [#{@link_state[@link]}]"
    end
  end
end
