require 'dread'

module Dread
  class Trace
    attr_reader :cpu
    attr_reader :data
    attr_reader :function
    attr_reader :pid
    attr_reader :task
    attr_reader :timestamp

    def initialize(line)
      @line = line
      @task = ''
      @pid = 0
      @cpu = 0
      @timestamp = 0
      @function = ''
      @data = ''

      parse!
    end

    def to_s
      "#{'%.9f' % timestamp}: #{task}-#{pid} [#{cpu}] #{function}: #{data}"
    end

    protected

    def parse!
      md = @line.match(re)

      raise "Can't parse '#{@line}'" unless md

      @cpu = md[:cpu].to_i
      @data = md[:data]
      @function = md[:function]
      @pid = md[:pid].to_i
      @task = md[:task]
      @timestamp = md[:timestamp].to_f
    end

    def re
      /(?<task>[a-z0-9\/-]+)-(?<pid>\d+)\s+\[(?<cpu>\d+)\]\s+....\s+(?<timestamp>[0-9.]+):\s+(?<function>\w+):\s+(?<data>.*)$/
    end

    attr_reader :line
  end
end
